#!/usr/bin/env python
import argparse

from siriuscommon.archiver import getPVsByDroppedEventsTypeChange

if __name__ == "__main__":
    parser = argparse.ArgumentParser("getPVsByDroppedEventsTypeChange Request")
    args = parser.parse_args()

    for pv in getPVsByDroppedEventsTypeChange():
        print(pv.pv_name, pv.events_dropped)



#!/usr/bin/env python3

from typing import Dict

import numpy
import pandas
from conscommon import get_logger
from conscommon.spreadsheet import SheetName

logger = get_logger("Parser")


def normalizeAgilent(sheet) -> dict:
    return normalize(sheet, ["C1", "C2", "C3", "C4"])


def normalizeMKS(sheet) -> dict:
    return normalize(sheet, ["A1", "A2", "B1", "B2", "C1", "C2"])


def normalizeMBTemp(sheet) -> dict:
    # ENABLE	IP	Rack	ADDR	Dev	Location
    return normalize(
        sheet,
        ["CH1", "CH2", "CH3", "CH4", "CH5", "CH6", "CH7", "CH8"],
        device_col="Dev",
    )


def normalize(
    sheet,
    ch_names: list,
    device_col: str = "Dispositivo",
    ip_col="IP",
    enable_col="ENABLE",
) -> dict:
    """Create a dictionary with the beaglebone IP as keys.  Aka: {ip:[devices ...] ... ipn:[devicesn ... ]}"""
    ips = {}
    try:
        for _n, row in sheet.iterrows():
            ip = row.get(ip_col)
            if ip not in ips:
                ips[ip] = []
            ip_devices = ips[ip]
            data = {}

            ip_devices.append(data)
            data["enable"] = row.get(enable_col, False)
            data["prefix"] = row.get(device_col)

            info = {}
            info["sector"] = row.get("Setor", "")
            info["serial_id"] = row.get("RS485 ID")
            info["rack"] = row.get("Rack")
            data["info"] = info

            channels = {}
            num = 0
            for ch_name in ch_names:
                channel = {}
                channel["num"] = num
                channel["prefix"] = row[ch_name]
                channel["enable"] = row[ch_name] != "" or row[ch_name] is None

                info = {}
                info["pressure_hi"] = row.get("HI " + ch_name)
                info["pressure_hihi"] = row.get("HIHI " + ch_name)
                if "Sensor " + ch_name in row:
                    info["sensor"] = row.get("Sensor " + ch_name)
                channel["info"] = info

                channels[ch_name] = channel
                num += 1

            data["channels"] = channels
    except Exception:
        logger.exception("Failed to update data from spreadsheet.")

    logger.info("Loaded data from sheet with {} different IPs.".format(len(ips)))
    return ips


def loadSheet(spreadsheet_xlsx_path: str, sheetName: SheetName) -> dict:
    logger.info(
        'Loading spreadsheet "{}" from url "{}"'.format(
            spreadsheet_xlsx_path, sheetName.value
        )
    )
    sheet = pandas.read_excel(spreadsheet_xlsx_path, sheet_name=sheetName.value)
    sheet = sheet.replace(numpy.nan, "", regex=True)

    if sheetName == SheetName.AGILENT:
        return normalizeAgilent(sheet)

    elif sheetName == SheetName.MKS:
        return normalizeMKS(sheet)

    elif sheetName == SheetName.MBTEMP:
        return normalizeMBTemp(sheet)

    else:
        return {}


def loadSheets(spreadsheet_xlsx_path: str) -> Dict[SheetName, dict]:
    data: Dict[SheetName, dict] = {}
    for sheetName in SheetName:
        data[sheetName] = loadSheet(spreadsheet_xlsx_path, sheetName)
    return data



import argparse as _argparse
import getpass as _getpass
import logging as _logging
import typing as _typing

__version__ = "2.0.1"
__author__ = "Carneiro, Claudio F."
__date__ = "Fri, 03 Sep 2021 16:30:02 +0000"


def get_logger(
    name=__file__,
    level: int = _logging.WARNING,
    handlers: _typing.Optional[_typing.List[_logging.Handler]] = None,
) -> _logging.Logger:
    """Returns a logger object"""

    logger = _logging.getLogger(name)

    if not len(logger.handlers) and not handlers:
        formatter = _logging.Formatter(
            "[%(asctime)s %(levelname)s %(filename)s:%(lineno)s - %(funcName)s] %(message)s"
        )
        logger.setLevel(level)
        console = _logging.StreamHandler()
        console.setFormatter(formatter)
        logger.addHandler(console)
    return logger


class PasswordPromptAction(_argparse.Action):
    """Custom password prompt action
    usage:

    >> parser = argparse.ArgumentParser(
        "Initialize a Taiga Project, ENG spreadsheet integration.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    >> parser.add_argument("-u", "--user", dest="user", type=str, required=True)
    >> parser.add_argument("-p", "--password", dest="password", action=PasswordPromptAction, type=str, required=True)

    >> args = parser.parse_args()"""

    def __init__(
        self,
        option_strings,
        dest=None,
        nargs=0,
        default=None,
        required=False,
        type=None,
        metavar=None,
        help=None,
    ):
        super(PasswordPromptAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            default=default,
            required=required,
            metavar=metavar,
            type=type,
            help=help,
        )

    def __call__(self, parser, args, values, option_string=None):
        password = _getpass.getpass()
        setattr(args, self.dest, password)



import typing as _typing
from datetime import datetime as _datetime


class ZabbixTrend(_typing.NamedTuple):
    data: dict
    timestamp: _datetime
    value_avg: str
    value_min: str
    value_max: str
    num: int


def _make_zabbix_trend(data: dict) -> ZabbixTrend:
    return ZabbixTrend(
        timestamp=_datetime.fromtimestamp(int(data["clock"])),
        value_avg=data["value_avg"],
        value_min=data["value_min"],
        value_max=data["value_max"],
        num=data["num"],
        data=data,
    )


class ZabbixHistory(_typing.NamedTuple):
    itemid: str
    value: str
    timestamp: _datetime
    data: dict


def _make_zabbix_history(data: dict):
    return ZabbixHistory(
        itemid=data["itemid"],
        value=data["value"],
        timestamp=_datetime.fromtimestamp(int(data["clock"])),
        data=data,
    )



import argparse as _argparse

from .. import PasswordPromptAction, get_logger
from .client import TaigaClient, _taiga_default_url

logger = get_logger("Taiga")


def taiga_export_xlsx():
    parser = _argparse.ArgumentParser(
        "Export a Taiga Project for ENG spreadsheet integration.",
        formatter_class=_argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "--slug",
        help=f"Project slug, the text located on your project URL. e.g.: {_taiga_default_url}/project/<project_slug> ",
        default="grupo-automacao-e-software",
    )
    parser.add_argument(
        "-o",
        "--output",
        dest="output",
        help="Output filename",
        default="user-stories.xlsx",
        required=False,
    )
    parser.add_argument(
        "--url", help="Taiga base URL", default=_taiga_default_url, required=False
    )
    parser.add_argument(
        "-u",
        "--username",
        dest="username",
        required=True,
        help="Taiga username with the required permissions.",
    )
    parser.add_argument(
        "-p",
        "--password",
        dest="password",
        action=PasswordPromptAction,
        type=str,
        required=True,
    )

    args = parser.parse_args()
    client = TaigaClient(host=args.url)
    logger.info("Authenticating...")
    client.auth(args.username, args.password)
    logger.info("Reading CSV...")
    df = client.get_stories_df(slug=args.slug)
    logger.info(f"Exporting to '{args.output}'")
    client.to_excel_table(df, filename=args.output)


def taiga_initialize_project():
    parser = _argparse.ArgumentParser(
        "Initialize a Taiga Project, ENG spreadsheet integration.",
        formatter_class=_argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "--slug",
        help=f"Project slug, the text located on your project URL. e.g.: {_taiga_default_url}/project/<project_slug> ",
        default="grupo-automacao-e-software",
    )
    parser.add_argument(
        "--url", help="Taiga base URL", default=_taiga_default_url, required=False
    )
    parser.add_argument(
        "-u",
        "--username",
        dest="username",
        required=True,
        help="Taiga username with the required permissions.",
    )
    parser.add_argument(
        "-p",
        "--password",
        dest="password",
        action=PasswordPromptAction,
        type=str,
        required=True,
    )

    args = parser.parse_args()

    client = TaigaClient(host=args.url)

    logger.info("Authenticating...")
    client.auth(args.username, args.password)

    logger.info("Initialising project")
    client.initialize_project(slug=args.slug)



import typing as _typing


class ChannelInfo:
    def __init__(self, *args, **kwargs):
        self.sensor = kwargs.get("sensor", "")
        self.pressure_high = kwargs.get("pressure_high", None)
        self.pressure_hihi = kwargs.get("pressure_hihi", None)


class Channel:
    def __init__(
        self,
        name: str = "",
        prefix: str = "",
        num: int = 0,
        info: dict = None,
        enable: bool = True,
        **kwargs
    ):
        if not info:
            info = {}

        self.name = name
        self.num = num
        self.prefix = prefix
        self.enable = enable
        self.info: _typing.Optional[ChannelInfo] = None
        if info:
            self.info = ChannelInfo(**info)

    def __repr__(self):
        return "{}(prefix={},name={})".format(
            self.__class__.__name__, self.prefix, self.name
        )


class DeviceInfo:
    def __init__(self, **kwargs):
        self.config = kwargs.get("config", "")
        self.rack = kwargs.get("rack", 0)
        self.sector = kwargs.get("sector", "")
        self.serial_id = kwargs.get("serial_id", -1)

    def __repr__(self):
        return "{}(config={})".format(self.__class__.__name__, self.config)


class Device:
    def __init__(
        self,
        prefix: str = "",
        info: dict = None,
        channels: dict = None,
        enable=True,
        **kwargs
    ):
        if not info:
            info = {}
        if not channels:
            channels = {}

        self.enable = enable
        self.info = info
        self.prefix = prefix
        self.channels: _typing.List[Channel] = []
        self._generateChannels(channels)

        self.info: _typing.Optional[DeviceInfo] = None
        if info:
            self.info = DeviceInfo(**info)

    def getChannelByPrefix(self, prefix: str) -> Channel:
        for channel in self.channels:
            if channel.prefix == prefix:
                return

    def _generateChannels(self, data):
        for k, v in data.items():
            if v.__class__.__name__ == Channel.__class__.__name__:
                self.channels.append(v)
            else:
                self.channels.append(Channel(name=k, **v))

    def __repr__(self):
        return "{}(prefix={},enabled={})".format(
            self.__class__.__name__, self.prefix, self.enable
        )


class Beaglebone:
    def __init__(self, ip: str = "", devices: list = None):

        if not devices:
            devices = []

        self.ip = ip
        self.devices: _typing.List[Device] = []
        self._generateDevices(devices)

    def _generateDevices(self, data):
        for d in data:
            if d.__class__.__name__ == Device.__class__.__name__:
                self.devices.append(d)
            else:
                self.devices.append(Device(**d))

    def __repr__(self):
        return "{}(ip={},devices={})".format(
            self.__class__.__name__, self.ip, self.devices
        )


def getDevicesFromBeagles(
    beagles: _typing.List[Beaglebone],
) -> _typing.List[Device]:
    """Generate devices from beagle"""
    devices: _typing.List[Device] = []
    for beagle in beagles:
        for device in beagle.devices:
            devices.append(device)
    return devices


def getDevicesFromList(data: list) -> _typing.List[Device]:
    """Load datamodel objects from a list of dictionaries"""
    devices: _typing.List[Device] = []
    for d in data:
        devices.append(Device(**d))
    return devices


def getBeaglesFromList(data: dict) -> _typing.List[Beaglebone]:
    """Load datamodel objects from dictionary"""
    beagles: _typing.List[Beaglebone] = []
    for k, v in data.items():
        beagle = Beaglebone(ip=k, devices=v)
        beagles.append(beagle)

    return beagles



#!/usr/bin/env python
import argparse

from siriuscommon.zabbix import ZabbixClient

if __name__ == "__main__":
    parser = argparse.ArgumentParser("List zabbix items from host")
    parser.add_argument("host", help="host name")
    parser.add_argument("--user", "-u", help="zabbix user", required=True)
    parser.add_argument("--password", "-p", help="zabbix password", required=True)
    args = parser.parse_args()
    host = args.host.strip()

    client = ZabbixClient(user=args.user, password=args.password)
    res = client.get_items_from_host(host)

    print("item.id; item.name; item.hostid; hostname; item.lastvalue")
    for item in res:
        print(f"{item.itemid}; {item.name}; {item.hostid}; {host}; {item.lastvalue}")



import typing as _typing
from argparse import ArgumentError

import requests as _requests

from .types import (
    ArchiverDisconnectedPV,
    ArchiverDroppedByTypeChangePV,
    ArchiverGenericReponse,
    ArchiverPausedPV,
    ArchiverSamplingMethod,
    ArchiverStatusPV,
    _make_archiver_disconnected_pv,
    _make_archiver_paused_pv,
    _make_archiver_status_pv,
)

_default_base_url = "https://10.0.38.42"


class ArchiverLoginException(Exception):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


class ArchiverClient:
    def __init__(
        self,
        base_url: str = _default_base_url,
        username: _typing.Optional[str] = None,
        password: _typing.Optional[str] = None,
    ) -> None:

        self._mgmt_url = f"{base_url}/mgmt/bpl"
        self._retrieval_url = f"{base_url}/retrieval/bpl"
        self._retrieval_data_url = f"{base_url}/retrieval/data"
        self._authenticated = False

        self._session: _requests.Session = _requests.Session()
        self._session.verify = False
        if username and password:
            self.login(username, password)

    @property
    def authenticated(self):
        return self._authenticated

    def login(self, username: str, password: str) -> ArchiverGenericReponse:
        data = {"username": username, "password": password}

        response = self._session.post(
            f"{self._mgmt_url}/login", data=data, verify=False
        )
        self._authenticated = (
            response.status_code == 200 and "authenticated" in response.text
        )
        if not self.authenticated:
            raise ArchiverLoginException(f"failed to authenticat user {username}")

        response_json = None
        try:
            response_json = response.json()
        except ValueError:
            response_json = None

        return ArchiverGenericReponse(
            success=self.authenticated,
            status_code=response.status_code,
            text=response.text,
            json=response_json,
        )

    def logout(self):
        raise NotImplementedError()

    def resume_pv(self, pv_name: str) -> ArchiverGenericReponse:
        raise NotImplementedError()

    def delete_pv(self, pv_name: str) -> ArchiverGenericReponse:
        raise NotImplementedError()

    def pause_pv(self, pv_name: str) -> ArchiverGenericReponse:
        if not self.authenticated:
            raise ArchiverLoginException("operation requires authentication")

        if not pv_name:
            raise ValueError()

        response = self._session.get(f"{self._mgmt_url}/pauseArchivingPV?pv={pv_name}")
        success = (
            response.status_code == 200
            and (f"Successfully paused the archiving of PV {pv_name}")
            or (f"PV {pv_name} is already paused") in response.text
        )
        response_json = None
        try:
            response_json = response.json()
        except ValueError:
            response_json = None

        return ArchiverGenericReponse(
            success=success,
            status_code=response.status_code,
            text=response.text,
            json=response_json,
        )

    def change_pv_archival_parameters(
        self, pv: str, sampling_method: ArchiverSamplingMethod, sampling_period: float
    ):
        """
        /changeArchivalParameters - Change the archival parameters for a PV.
        pv: The name of the pv.
        samplingperiod: The new sampling period in seconds.
        samplingmethod: The new sampling method For now, this is one of SCAN or MONITOR.
        """
        if sampling_method < 0.01:
            raise ArgumentError(f"'sampling_period' {sampling_period} invalue range")
        if (
            type(sampling_method) != str
            and type(sampling_method) != ArchiverSamplingMethod
        ):
            raise ArgumentError(f"'sampling_method' {sampling_method} invalid type")
        if (
            sampling_method != ArchiverSamplingMethod.SCAN
            and sampling_method != ArchiverSamplingMethod.MONITOR
        ):
            raise ArgumentError(f"'sampling_method' {sampling_method} invalid option")

        # sampling_method_value = sampling_method.value

        # /changeArchivalParameters
        raise NotImplementedError()

    def get_pv_details(self, pv_name: str):
        raise NotImplementedError()

    def get_currently_disconnected_pvs(self) -> _typing.List[ArchiverDisconnectedPV]:
        return [
            _make_archiver_disconnected_pv(**pv)
            for pv in self._session.get(
                f"{self._mgmt_url}/getCurrentlyDisconnectedPVs", verify=False
            ).json()
        ]

    def get_paused_pvs_report(self) -> _typing.List[ArchiverPausedPV]:
        return [
            _make_archiver_paused_pv(**pv)
            for pv in self._session.get(
                f"{self._mgmt_url}/getPausedPVsReport", verify=False
            ).json()
        ]

    def get_matching_pvs(self, search: str, limit: int = 500) -> _typing.List[str]:
        return [
            pv
            for pv in self._session.get(
                f"{self._retrieval_url}/getMatchingPVs",
                params={"pv": search, "limit": limit},
                verify=False,
            ).json()
        ]

    def get_pv_status(self, search: str, reporttype: str = "short") -> ArchiverStatusPV:
        return [
            _make_archiver_status_pv(**pv)
            for pv in self._session.get(
                f"{self._mgmt_url}/getPVStatus",
                params={"pv": search, "reporttype": reporttype},
                verify=False,
            ).json()
        ]

    def get_pvs_by_dropped_events_type_change(
        self, limit: int = 200
    ) -> _typing.List[ArchiverDroppedByTypeChangePV]:
        return [
            ArchiverDroppedByTypeChangePV(
                pv_name=pv["pvName"], events_dropped=pv["eventsDropped"]
            )
            for pv in self._session.get(
                f"{self._mgmt_url}/getPVsByDroppedEventsTypeChange",
                params={limit: limit},
            ).json()
        ]

    def change_type_for_pv(self):
        raise NotImplementedError()


def getPVsByDroppedEventsTypeChange(
    base_url: str = _default_base_url,
) -> _typing.List[ArchiverDroppedByTypeChangePV]:
    return ArchiverClient(base_url=base_url).get_pvs_by_dropped_events_type_change()


def getCurrentlyDisconnectedPVs(
    base_url: str = _default_base_url,
) -> _typing.List[ArchiverDisconnectedPV]:
    return ArchiverClient(base_url=base_url).get_currently_disconnected_pvs()


def getPausedPVsReport(
    base_url: str = _default_base_url,
) -> _typing.List[ArchiverPausedPV]:
    return ArchiverClient(base_url=base_url).get_paused_pvs_report()


def getMatchingPVs(search: str, base_url: str = _default_base_url) -> _typing.List[str]:
    return ArchiverClient(base_url=base_url).get_matching_pvs(search=search)


def getPVStatus(search: str, base_url: str = _default_base_url):
    return ArchiverClient(base_url=base_url).get_pv_status(search=search)
